setwd("<YOUR_DIRECTORY>")

# read data
data <- read.csv(file="data/java_distinct_a_so-data_cb.csv", header=TRUE, sep=",", quote="\"", na.strings=c(""), stringsAsFactors=FALSE)

# delete currupted rows
data <- data[!is.na(data$question_title),]
# if owner has no reputation, it is NA => set it to 0
data[is.na(data$answer_owner_reputation),]$answer_owner_reputation <- 0
#data[is.na(data$question_owner_reputation),]$question_owner_reputation <- 0
# assign age to questions
now <- as.POSIXct("2016-07-01 UTC", tz="UTC", origin="1970-01-01")
data$question_creation_date <- as.POSIXct(data$question_creation_date, tz="UTC", origin="1970-01-01")
question_age <- difftime(now, data$question_creation_date, units="days")
data[["question_age"]] <- question_age
data$answer_creation_date <- as.POSIXct(data$answer_creation_date, tz="UTC", origin="1970-01-01")
answer_age <- difftime(now, data$answer_creation_date, units="days")
data[["answer_age"]] <- answer_age

#answers_filter <- grepl("http://stackoverflow.com/a/[0-9]+", data$url, per=TRUE)
#questions_filter <- grepl("http://stackoverflow.com/q/[0-9]+", data$url, per=TRUE)

nrow(data)
# => 2433 (2443 with currupted rows)

summary(data$line_count)
# Min. 1st Qu.  Median    Mean 3rd Qu.    Max. 
# 1.000   1.000   1.000   1.323   1.000  21.000

table(data$line_count)
#   1    2    3    4    5    6    7    8   12   19   20   21 
#1975  306   81   41   13    4    7    2    1    1    1    1 

barplot(table(data$line_count))

d <- density(data$line_count)
plot(d)
polygon(d, col="gray", border="black")

quantile(data$line_count, c(0.8, 0.95))
# 80% 95% 
#  1   3
length(data$line_count[data$line_count<=1])/length(data$line_count)*100
# 81.1755
length(data$line_count[data$line_count<=3])/length(data$line_count)*100
# 97.08179

barplot(sort(data$question_view_count, decreasing=TRUE))
barplot(sort(data$question_score, decreasing=TRUE))
barplot(sort(data$answer_score, decreasing=TRUE))
barplot(sort(as.numeric(data$question_age), decreasing=TRUE))
barplot(sort(as.numeric(data$answer_age), decreasing=TRUE))

lc_1 <- data[data$line_count==1,]
nrow(lc_1) # 1975
barplot(sort(lc_1$question_view_count, decreasing=TRUE))
barplot(sort(lc_1$question_score, decreasing=TRUE))
barplot(sort(lc_1$answer_score, decreasing=TRUE))
barplot(sort(as.numeric(lc_1$question_age), decreasing=TRUE))
barplot(sort(as.numeric(lc_1$answer_age), decreasing=TRUE))

lc_2_3 <- data[data$line_count==2 | data$line_count==3,]
nrow(lc_2_3) # 387
barplot(sort(lc_2_3$question_view_count, decreasing=TRUE))
barplot(sort(lc_2_3$question_score, decreasing=TRUE))
barplot(sort(lc_2_3$answer_score, decreasing=TRUE))
barplot(sort(as.numeric(lc_2_3$question_age), decreasing=TRUE))
barplot(sort(as.numeric(lc_2_3$answer_age), decreasing=TRUE))

lc_4 <- data[data$line_count>3,]
nrow(lc_4) # 71
barplot(sort(lc_4$question_view_count, decreasing=TRUE))
barplot(sort(lc_4$question_score, decreasing=TRUE))
barplot(sort(lc_4$answer_score, decreasing=TRUE))
barplot(sort(as.numeric(lc_4$question_age), decreasing=TRUE))
barplot(sort(as.numeric(lc_4$answer_age), decreasing=TRUE))

library(vioplot)
vioplot(lc_4$question_view_count,
        lc_2_3$question_view_count,
        lc_1$question_view_count,
        data$question_view_count,
        col="gray")
vioplot(lc_4$question_score,
        lc_2_3$question_score,
        lc_1$question_score,
        data$question_score,
        col="gray")
vioplot(lc_4$answer_score,
        lc_2_3$answer_score,
        lc_1$answer_score,
        data$answer_score,
        col="gray")
vioplot(lc_4$question_age,
        lc_2_3$question_age,
        lc_1$question_age,
        data$question_age,
        col="gray")
vioplot(lc_4$answer_age,
        lc_2_3$answer_age,
        lc_1$answer_age,
        data$answer_age,
        col="gray")

barplot(sort(lc_4$question_view_count, decreasing=TRUE))
barplot(sort(lc_4$question_score, decreasing=TRUE))
barplot(sort(lc_4$answer_score, decreasing=TRUE))
barplot(sort(as.numeric(lc_4$question_age), decreasing=TRUE))
barplot(sort(as.numeric(lc_4$answer_age), decreasing=TRUE))


# Correlations
corr_vars <- data[,c(2, 7, 10, 22, 32, 33)]
corr_vars$question_age <- as.numeric(corr_vars$question_age)
corr_vars$answer_age <- as.numeric(corr_vars$answer_age)

library(corrgram)
corrgram(corr_vars, order=FALSE, lower.panel=panel.shade, upper.panel=panel.conf, cor.method="pearson")

corr_vars_age_norm <- corr_vars
corr_vars_age_norm$question_view_count <- corr_vars$question_view_count/corr_vars$question_age
corr_vars_age_norm$question_score <- corr_vars$question_score/corr_vars$question_age
corr_vars_age_norm$answer_score <- corr_vars$answer_score/corr_vars$answer_age

corrgram(corr_vars_age_norm, order=FALSE, lower.panel=panel.shade, upper.panel=panel.conf, cor.method="pearson")

corr_vars_lc_4 <- lc_4[,c(2, 7, 10, 22, 32, 33)]
corr_vars_lc_4$question_age <- as.numeric(corr_vars_lc_4$question_age)
corr_vars_lc_4$answer_age <- as.numeric(corr_vars_lc_4$answer_age)
corrgram(corr_vars_lc_4, order=FALSE, lower.panel=panel.shade, upper.panel=panel.conf, cor.method="pearson")

library(Hmisc) # for rcorr
rcorr(as.matrix(corr_vars), type="pearson") # pariwise deletion
rcorr(as.matrix(corr_vars_lc_4), type="pearson") # pariwise deletion

# see http://stats.stackexchange.com/a/70807, http://stats.stackexchange.com/a/180249, http://stackoverflow.com/a/15779107
normalize <- function(x, min_x, max_x) {
  return ((x-min_x) / (max_x-min_x))
}

normalize_df <- function(df) {
  result <- NULL
  for (i in names(df)) {
    min_i <- min(df[,i])
    max_i <- max(df[,i])
    norm_i <- unlist(as.numeric(lapply(df[,i], function(x) {
      normalize(x, min_i, max_i)
    })))
    if (is.null(result)) {
      result <- norm_i
    } else {
      result <- cbind(result, norm_i)
    }
  }
  result <- data.frame(result)
  names(result) <- names(df)
  return(result)
}

corr_vars_norm <- normalize_df(corr_vars)
library(car)
scatterplotMatrix(~corr_vars_norm$line_count+corr_vars_norm$question_view_count+corr_vars_norm$question_score+corr_vars_norm$answer_score+corr_vars_norm$question_age+corr_vars_norm$answer_age, data=corr_vars_norm)

corr_vars_lc_4_norm <- normalize_df(corr_vars_lc_4)
scatterplotMatrix(~corr_vars_lc_4_norm$line_count+corr_vars_lc_4_norm$question_view_count+corr_vars_lc_4_norm$question_score+corr_vars_lc_4_norm$answer_score+corr_vars_lc_4_norm$question_age+corr_vars_lc_4_norm$answer_age, data=corr_vars_lc_4_norm)

# answer age
summary(as.numeric(data$answer_age)/365)
# Min.    1st Qu. Median  Mean    3rd Qu. Max. 
# 0.1097  2.7030  4.0150  4.0590  5.4010  7.8700

vioplot(data$answer_age/365,
        col="gray90", pchMed=4, rectCol="white", colMed="black", horizontal=TRUE, wex=1.15,
        names=c(""), ylim=c(0, max(data$question_age/365)))  # use same limits as question age
abline(v=mean(data$answer_age/365), lty=3)
title("answer age (years)")


# analyze code blocks
data$answer_code_blocks <- lapply(data$answer_code_blocks, function(x) {
    trimws(unlist(strsplit(x, '-', fixed=TRUE)))
})

code_block_count <- unlist(lapply(data$answer_code_blocks, function(x) {
    as.integer(x[1])
}))

summary(code_block_count)
# Min. 1st Qu.  Median    Mean 3rd Qu.    Max. 
# 0.000   1.000   2.000   4.088   4.000 203.000 

high_ref <-data$line_count>4
low_ref <- data$line_count<=4

summary(code_block_count[high_ref])
# Min. 1st Qu.  Median    Mean 3rd Qu.    Max. 
# 1.000   2.000   3.000   9.533   7.750  94.000 

summary(code_block_count[low_ref])
# Min. 1st Qu.  Median    Mean 3rd Qu.    Max. 
# 0.00    1.00    2.00    4.02    4.00  203.00 

table(code_block_count)[1:20]
# 0   1   2   3   4   5   6   7   8   9  10  11  12  13  14  15  16  17  18  19 
# 113 819 462 259 185 115  81  57  66  43  27  26  28  25  18  16   6  11  10   8 

vioplot(code_block_count)

semicolon <- unlist(lapply(data$answer_code_blocks, function(x) {
    as.integer(x[2])
}))

table(semicolon)
# 0    1 
# 295 2138
