setwd("<YOUR_DIRECTORY>")

### READ DATA ###
q_mapped <- read.csv(file="data/global_q_line_count_mapped_so-data.csv", header=TRUE, sep=",", quote="\"", na.strings=c(""), stringsAsFactors=FALSE) 

java_q_mapped <- q_mapped[q_mapped$language=="java",]
java_q_mapped_10 <- java_q_mapped[java_q_mapped$line_count>9,]

### CLEAN DATA ###
# delete currupted rows
nrow(java_q_mapped_10)
# 79
java_q_mapped_10 <- java_q_mapped_10[!is.na(java_q_mapped_10$question_title),]
nrow(java_q_mapped_10)
# 79
# if owner has no reputation, it is NA => set it to 0
if (nrow(java_q_mapped_10[is.na(java_q_mapped_10$question_owner_reputation),])>0) {
  java_q_mapped_10[is.na(java_q_mapped_10$question_owner_reputation),]$question_owner_reputation <- 0  
}
# assign age to questions
now <- as.POSIXct("2016-08-01 UTC", tz="UTC", origin="1970-01-01")
java_q_mapped_10$question_creation_date <- as.POSIXct(java_q_mapped_10$question_creation_date, tz="UTC", origin="1970-01-01")
question_age <- difftime(now, java_q_mapped_10$question_creation_date, units="days")
java_q_mapped_10[["question_age"]] <- question_age

java_q_mapped_10$question_tags <- lapply(java_q_mapped_10$question_tags, function(x) {
  unique(trimws(unlist(strsplit(x, ';', fixed=TRUE))))
})

unique_tags <- unique(unlist(java_q_mapped_10$question_tags))
tag_count <- sort(table(unlist((java_q_mapped_10$question_tags))), decreasing=TRUE)
tag_count[tag_count>2]
# java; android;    file;  string; file-io; 
# 53       27        6        6        3 

barplot(tag_count)

java_tagged <- unlist(lapply(java_q_mapped_10$question_tags, function(x) {
  'java' %in% x
}))

# write files not tagged as java
java_q_mapped_10$question_tags <- unlist(lapply(java_q_mapped_10$question_tags, function(x) {
  paste(x , collapse = ' ')
}))
write.table(java_q_mapped_10[!java_tagged,], file = paste("data/java_not_tagged.csv", sep=""), sep=",", col.names=TRUE, row.names=FALSE, na="", quote=FALSE)

