setwd("<YOUR_DIRECTORY>")

### READ DATA ###
q <- read.csv(file="data/global_q_line_count_so-data.csv", header=TRUE, sep=",", quote="\"", na.strings=c(""), stringsAsFactors=FALSE) 
q_mapped <- read.csv(file="data/global_q_line_count_mapped_so-data.csv", header=TRUE, sep=",", quote="\"", na.strings=c(""), stringsAsFactors=FALSE) 
q_2_9 <- read.csv(file="data/global_q_line_count_2-9_so-data.csv", header=TRUE, sep=",", quote="\"", na.strings=c(""), stringsAsFactors=FALSE) 
q_10 <- read.csv(file="data/global_q_line_count_10_so-data.csv", header=TRUE, sep=",", quote="\"", na.strings=c(""), stringsAsFactors=FALSE)

### CLEAN DATA ###
# delete currupted rows
nrow(q)
# 36041
q <- q[!is.na(q$question_title),]
nrow(q)
# 35882
nrow(q_mapped)
# 44263
q_mapped <- q_mapped[!is.na(q_mapped$question_title),]
nrow(q_mapped)
# 44089
nrow(q_2_9)
# 8294
q_2_9 <- q_2_9[!is.na(q_2_9$question_title),]
nrow(q_2_9)
# 8257
nrow(q_10)
# 383
q_10 <- q_10[!is.na(q_10$question_title),]
nrow(q_10)
# 382
# if owner has no reputation, it is NA => set it to 0
if (nrow(q[is.na(q$question_owner_reputation),])>0) {
  q[is.na(q$question_owner_reputation),]$question_owner_reputation <- 0  
}
if (nrow(q_mapped[is.na(q_mapped$question_owner_reputation),])>0) {
  q_mapped[is.na(q_mapped$question_owner_reputation),]$question_owner_reputation <- 0  
}
if (nrow(q_2_9[is.na(q_2_9$question_owner_reputation),])>0) {
    q_2_9[is.na(q_2_9$question_owner_reputation),]$question_owner_reputation <- 0  
}
if (nrow(q_10[is.na(q_10$question_owner_reputation),])>0) {
    q_10[is.na(q_10$question_owner_reputation),]$question_owner_reputation <- 0  
}
# assign age to questions
now <- as.POSIXct("2016-08-01 UTC", tz="UTC", origin="1970-01-01")
q$question_creation_date <- as.POSIXct(q$question_creation_date, tz="UTC", origin="1970-01-01")
question_age <- difftime(now, q$question_creation_date, units="days")
q[["question_age"]] <- question_age
q_mapped$question_creation_date <- as.POSIXct(q_mapped$question_creation_date, tz="UTC", origin="1970-01-01")
question_age <- difftime(now, q_mapped$question_creation_date, units="days")
q_mapped[["question_age"]] <- question_age
q_2_9$question_creation_date <- as.POSIXct(q_2_9$question_creation_date, tz="UTC", origin="1970-01-01")
question_age <- difftime(now, q_2_9$question_creation_date, units="days")
q_2_9[["question_age"]] <- question_age
q_10$question_creation_date <- as.POSIXct(q_10$question_creation_date, tz="UTC", origin="1970-01-01")
question_age <- difftime(now, q_10$question_creation_date, units="days")
q_10[["question_age"]] <- question_age

length(which(q$line_count==1))
# 27245
27245/nrow(q)*100
# 75.92944

summary(q$line_count)
# Min. 1st Qu.  Median    Mean 3rd Qu.    Max. 
# 1.000   1.000   1.000   1.624   1.000 228.000 
summary(q_mapped$line_count)
# Min. 1st Qu.  Median    Mean 3rd Qu.    Max. 
# 1.000   1.000   1.000   1.831   2.000 474.000 
summary(q_10$line_count)
# Min. 1st Qu.  Median    Mean 3rd Qu.    Max. 
# 10.00   11.00   14.00   19.05   21.00  228.00
summary(q_2_9$line_count)
# Min. 1st Qu.  Median    Mean 3rd Qu.    Max. 
# 2.000   2.000   2.000   2.876   3.000   9.000 
table(q_2_9$line_count)

barplot(table(q_2_9$line_count))
barplot(table(q_10$line_count))

library(scales)  # for color transparency (alpha) 

d_2_9 <- density(q_2_9$question_view_count) 
d_10 <- density(q_10$question_view_count)
plot(d_2_9)
lines(d_10)
polygon(d_10, col=alpha("gray90", 0.5), border="black")
polygon(d_2_9, col=alpha("gray70", 0.5), border="black")

d_2_9 <- density(q_2_9$question_score) 
d_10 <- density(q_10$question_score)
plot(d_2_9)
lines(d_10)
polygon(d_10, col=alpha("gray90", 0.5), border="black")
polygon(d_2_9, col=alpha("gray70", 0.5), border="black")

d_2_9 <- density(as.numeric(q_2_9$question_age)) 
d_10 <- density(as.numeric(q_10$question_age))
plot(d_10)
lines(d_2_9)
polygon(d_10, col=alpha("gray90", 0.5), border="black")
polygon(d_2_9, col=alpha("gray70", 0.5), border="black")

library(vioplot)

pdf("figures/violin_q_global.pdf", bg="transparent", width=4, height=7)
par(
    mar = c(2.5, 1.5, 1.5, 1.5)+0.1, # subplot margins (bottom, left, top, right)
    omi = c(0.2, 0.2, 0.2, 0.2),  # outer margins in inches (bottom, left, top, right)
    mfrow = c(3, 1)
    #pin = (width, height)
    #mfcol # draw in columns
)

vioplot(q_2_9$question_view_count,
        q_10$question_view_count,
        col="gray90", pchMed=4, rectCol="white", colMed="black", horizontal=TRUE, wex=1.15,
        names=c("2-9 ref. lines", ">9 ref. lines"), ylim=c(0, max(q_10$question_view_count)))
#lines(data.frame(c(1, 2), c(median(q_10$question_view_count), median(q_10$question_view_count))), lty=3)
abline(v=median(q_10$question_view_count), lty=3)
title("question view count", cex.main=1.1)

vioplot(q_2_9$question_score,
        q_10$question_score,
        col="gray90", pchMed=4, rectCol="white", colMed="black", horizontal=TRUE, wex=1.15,
        names=c("2-9 ref. lines", ">9 ref. lines"), ylim=c(0, max(q_10$question_score)))
#lines(data.frame(c(1, 2), c(median(q_10$question_score), median(q_10$question_score))), lty=3)
abline(v=median(q_10$question_score), lty=3)
title("question score", cex.main=1.1)

vioplot(q_2_9$question_age,
        q_10$question_age,
        col="gray90", pchMed=4, rectCol="white", colMed="black", horizontal=TRUE, wex=1.15,
        names=c("2-9 ref. lines", ">9 ref. lines"), ylim=c(0, max(q_10$question_age)))
#lines(data.frame(c(1, 2), c(median(q_10$question_age), median(q_10$question_age))), lty=3)
abline(v=median(q_10$question_age), lty=3)
title("question age", cex.main=1.1)

par(mfrow = c(1, 1))
dev.off() 

### T-TEST ###
# http://stats.stackexchange.com/questions/31326/how-should-one-interpret-the-comparison-of-means-from-different-sample-sizes/31330#31330
# http://stats.stackexchange.com/questions/82705/unequal-sample-sizes-when-to-call-it-quits
#t.test(q_10$question_view_count, q_2_9$question_view_count, alternative="greater", paired=FALSE, conf.level=0.99, var.equal=FALSE)
# http://math.stackexchange.com/questions/625359/how-are-the-values-in-the-shapiro-wilk-weight-table-calculated
# http://stats.stackexchange.com/questions/15696/interpretation-of-shapiro-wilk-test/15702#15702
# http://www.r-bloggers.com/normality-tests-don%E2%80%99t-do-what-you-think-they-do/
# http://stackoverflow.com/questions/15427692/perform-a-shapiro-wilk-normality-test/15427746#15427746
# http://stackoverflow.com/questions/7781798/seeing-if-data-is-normally-distributed-in-r/7788452#7788452
#shapiro.test(q_10$question_view_count)
#shapiro.test(q_2_9$question_view_count)

### WILCOXON TEST AND EFFECT SIZE ###
library(effsize)

summary(q_10$question_view_count)
# Min. 1st Qu.  Median    Mean 3rd Qu.    Max. 
# 824   50220  121000  234500  289700 2245000 
sd(q_10$question_view_count)
# 312393
summary(q_2_9$question_view_count)
# Min. 1st Qu.  Median    Mean 3rd Qu.    Max. 
#  16    6276   20820   70040   66730 3467000 
sd(q_2_9$question_view_count)
# 163695.3
wilcox.test(q_10$question_view_count, q_2_9$question_view_count, alternative="greater", paired=F, correct=T) # Wilcoxon rank sum test (equivalent to the Mann-Whitney test), exact distribution (Streitberg-Roehmel algorithm) => handles ties
# W = 2484920, p-value < 2.2e-16
# alternative hypothesis: true location shift is greater than 0
median(q_10$question_view_count)-median(q_2_9$question_view_count)
cliff.delta(q_10$question_view_count, q_2_9$question_view_count)
# delta estimate: 0.5756391 (large)
# 95 percent confidence interval:
#   inf       sup 
# 0.5316636 0.6165193


summary(q_10$question_score)
# Min. 1st Qu.  Median    Mean 3rd Qu.    Max. 
# 1.0    72.0   171.0   357.4   404.5  3028.0 
sd(q_10$question_score)
# 506.7109
summary(q_2_9$question_score)
# Min. 1st Qu.  Median    Mean 3rd Qu.    Max. 
# -3.0     9.0    27.0   100.1    76.0 15170.0 
sd(q_2_9$question_score)
# 370.7056
wilcox.test(q_10$question_score, q_2_9$question_score, alternative="greater", paired=F, correct=T) # Wilcoxon rank sum test (equivalent to the Mann-Whitney test), exact distribution (Streitberg-Roehmel algorithm) => handles ties
# W = 2549754, p-value < 2.2e-16
# alternative hypothesis: true location shift is greater than 0
median(q_10$question_score)-median(q_2_9$question_score)
cliff.delta(q_10$question_score, q_2_9$question_score)
# delta estimate: 0.6167488 (large)
# 95 percent confidence interval:
#   inf       sup 
# 0.5727301 0.6572209 

summary(as.numeric(q_10$question_age)/365)
# Min. 1st Qu.  Median    Mean 3rd Qu.    Max. 
# 1.245   5.385   6.638   6.331   7.485   7.987 
sd(as.numeric(q_10$question_age)/365)
# 1.360693
summary(as.numeric(q_2_9$question_age)/365)
# Min. 1st Qu.  Median    Mean 3rd Qu.    Max. 
# 0.0797  3.9410  5.3410  5.2120  6.6060  8.0060
sd(as.numeric(q_2_9$question_age)/365)
# 1.698796
wilcox.test(as.numeric(q_10$question_age)/365, as.numeric(q_2_9$question_age)/365, alternative="greater", paired=F, correct=T) # Wilcoxon rank sum test (equivalent to the Mann-Whitney test), exact distribution (Streitberg-Roehmel algorithm) => handles ties
# W = 2190240, p-value < 2.2e-16
# alternative hypothesis: true location shift is greater than 0
median(as.numeric(q_10$question_age)/365)-median(as.numeric(q_2_9$question_age)/365)
cliff.delta(q_10$question_age/365, q_2_9$question_age/365)
# delta estimate: 0.388788 (medium)
# 95 percent confidence interval:
#   inf       sup 
# 0.3364700 0.4387133 


# Correlations
library(corrgram)
corr_vars_q <- q[,c(3, 8, 11, 33)]
corr_vars_q$question_age <- as.numeric(corr_vars_q$question_age)
corrgram(corr_vars_q, order=FALSE, lower.panel=panel.shade, upper.panel=panel.conf, cor.method="spearman")

corr_vars_q_mapped <- q_mapped[,c(3, 8, 11, 33)]
corr_vars_q_mapped$question_age <- as.numeric(corr_vars_q_mapped$question_age)
corrgram(corr_vars_q_mapped, order=FALSE, lower.panel=panel.shade, upper.panel=panel.conf, cor.method="spearman")

corr_vars_q_10 <- q_10[,c(3, 8, 11, 33)]
corr_vars_q_10$question_age <- as.numeric(corr_vars_q_10$question_age)
corrgram(corr_vars_q_10, order=FALSE, lower.panel=panel.shade, upper.panel=panel.conf, cor.method="spearman")

corr_vars_q_2_9 <- q_2_9[,c(3, 8, 11, 33)]
corr_vars_q_2_9$question_age <- as.numeric(corr_vars_q_2_9$question_age)
corrgram(corr_vars_q_2_9, order=FALSE, lower.panel=panel.shade, upper.panel=panel.conf, cor.method="spearman")

corr_vars_q_10_age_norm <- corr_vars_q_10
corr_vars_q_10_age_norm$question_view_count <- corr_vars_a_10$question_view_count/corr_vars_a_10$question_age
corr_vars_q_10_age_norm$question_score <- corr_vars_a_10$question_score/corr_vars_a_10$question_age
corrgram(corr_vars_q_10_age_norm, order=FALSE, lower.panel=panel.shade, upper.panel=panel.conf, cor.method="spearman")

corr_vars_q_2_9_age_norm <- corr_vars_q_2_9
corr_vars_q_2_9_age_norm$question_view_count <- corr_vars_q_2_9$question_view_count/corr_vars_q_2_9$question_age
corr_vars_q_2_9_age_norm$question_score <- corr_vars_q_2_9$question_score/corr_vars_q_2_9$question_age
corr_vars_q_2_9_age_norm$answer_score <- corr_vars_q_2_9$answer_score/corr_vars_q_2_9$answer_age
corrgram(corr_vars_q_2_9_age_norm, order=FALSE, lower.panel=panel.shade, upper.panel=panel.conf, cor.method="spearman")

get_cor_p_values <- function(df) {
  result <- data.frame(matrix(ncol=ncol(df), nrow=ncol(df)))
  if (ncol(df) < 2) {
    return(result)
  }
  names(result) <- names(df)
  row.names(result) <- names(df)
  for (i in 1:ncol(df)) {
    for (j in 2:ncol(df)) {
      if (i==j) {
        result[i,j] <- NA  
      } else {
        result[i,j] <- cor.test(df[,i], df[,j], alternative="two.sided", method="spearman", exact=FALSE)$p.value
        result[j,i] <- result[i,j]
      }
    }
  }
  return(result)
}

get_cor_n <- function(df) {
  result <- data.frame(matrix(ncol=ncol(df), nrow=ncol(df)))
  if (ncol(df) < 2) {
    return(result)
  }
  names(result) <- names(df)
  row.names(result) <- names(df)
  for (i in 1:ncol(df)) {
    for (j in 2:ncol(df)) {
      if (i==j) {
        result[i,j] <- NA  
      } else {
        comp_cases <- complete.cases(cbind(df[,i], df[,j]))
        result[i,j] <- length(comp_cases[comp_cases==TRUE])
        result[j,i] <- result[i,j]
      }
    }
  }
  return(result)
}

cor(corr_vars_q, use="pairwise.complete.obs", method="spearman")
#                      line_count  question_view_count question_score question_age
# line_count           1.0000000           0.2667161      0.2972203    0.1839781
# question_view_count  0.2667161           1.0000000      0.8503900    0.5145331
# question_score       0.2972203           0.8503900      1.0000000    0.4936940
# question_age         0.1839781           0.5145331      0.4936940    1.0000000
get_cor_p_values(corr_vars_q)
#                        line_count question_view_count question_score  question_age
# line_count                     NA                   0              0 1.189242e-270
# question_view_count  0.000000e+00                  NA              0  0.000000e+00
# question_score       0.000000e+00                   0             NA  0.000000e+00
# question_age        1.189242e-270                   0              0            NA
get_cor_n(corr_vars_q)
#                        line_count question_view_count question_score question_age
# line_count                  NA               35882          35882        35882
# question_view_count      35882                  NA          35882        35882
# question_score           35882               35882             NA        35882
# question_age             35882               35882          35882           NA

cor(corr_vars_q_mapped, use="pairwise.complete.obs", method="spearman")
#                      line_count   question_view_count question_score question_age
# line_count           1.0000000           0.2803433      0.3137938    0.1880476
# question_view_count  0.2803433           1.0000000      0.8470482    0.5153906
# question_score       0.3137938           0.8470482      1.0000000    0.4909279
# question_age         0.1880476           0.5153906      0.4909279    1.0000000
get_cor_p_values(corr_vars_q_mapped)
#                         line_count question_view_count question_score question_age
# line_count                  NA                   0              0            0
# question_view_count          0                  NA              0            0
# question_score               0                   0             NA            0
# question_age                 0                   0              0           NA
get_cor_n(corr_vars_q_mapped)
#                       line_count question_view_count question_score question_age
# line_count                  NA               44089          44089        44089
# question_view_count      44089                  NA          44089        44089
# question_score           44089               44089             NA        44089
# question_age             44089               44089          44089           NA


# see http://stats.stackexchange.com/a/70807, http://stats.stackexchange.com/a/180249, http://stackoverflow.com/a/15779107
normalize <- function(x, min_x, max_x) {
    return ((x-min_x) / (max_x-min_x))
}
normalize_df <- function(df) {
    result <- NULL
    for (i in names(df)) {
        min_i <- min(df[,i])
        max_i <- max(df[,i])
        norm_i <- unlist(as.numeric(lapply(df[,i], function(x) {
            normalize(x, min_i, max_i)
        })))
        if (is.null(result)) {
            result <- norm_i
        } else {
            result <- cbind(result, norm_i)
        }
    }
    result <- data.frame(result)
    names(result) <- names(df)
    return(result)
}

library(car)
corr_vars_q_10_norm <- normalize_df(corr_vars_q_10)
scatterplotMatrix(~corr_vars_q_10_norm$line_count+corr_vars_q_10_norm$question_view_count+corr_vars_q_10_norm$question_score+corr_vars_q_10_norm$answer_score+corr_vars_q_10_norm$question_age+corr_vars_q_10_norm$answer_age, data=corr_vars_q_10_norm)

corr_vars_q_2_9_norm <- normalize_df(corr_vars_q_2_9)
scatterplotMatrix(~corr_vars_q_2_9_norm$line_count+corr_vars_q_2_9_norm$question_view_count+corr_vars_q_2_9_norm$question_score+corr_vars_q_2_9_norm$answer_score+corr_vars_q_2_9_norm$question_age+corr_vars_q_2_9_norm$answer_age, data=corr_vars_q_2_9_norm)

plot(corr_vars_q_10$line_count, corr_vars_q_10$question_score)
nrow(corr_vars_q_10)
corr_vars_q_10_new <- corr_vars_q_10[corr_vars_q_10$line_count<=60,]
nrow(corr_vars_q_10_new)
plot(corr_vars_q_10_new$line_count, corr_vars_q_10_new$question_score)

corr_vars_q_10_new_norm <- normalize_df(corr_vars_q_10_new)
fitted_model <- lm(corr_vars_q_10_new_norm$line_count~corr_vars_q_10_new_norm$question_score)
summary(fitted_model)
plot(corr_vars_q_10_new_norm$line_count, corr_vars_q_10_new_norm$question_score)
abline(fitted_model, col="red")

plot(corr_vars_q_2_9$line_count, corr_vars_q_2_9$question_score)
corr_vars_q_2_9_norm <- normalize_df(corr_vars_q_2_9)
fitted_model <- lm(corr_vars_q_2_9_norm$line_count~corr_vars_q_2_9_norm$question_score)
summary(fitted_model)
plot(corr_vars_q_2_9_norm$line_count, corr_vars_q_2_9_norm$question_score)
abline(fitted_model, col="red")


