setwd("<YOUR_DIRECTORY>")

# read data
data <- read.csv(file="data/java_distinct_q_so-data.csv", header=TRUE, sep=",", quote="\"", na.strings=c(""), stringsAsFactors=FALSE)
# delete currupted rows
data <- data[!is.na(data$question_title),]
# if owner has no reputation, it is NA => set it to 0
data[is.na(data$answer_owner_reputation),]$answer_owner_reputation <- 0
#data[is.na(data$question_owner_reputation),]$question_owner_reputation <- 0
# assign age to questions
now <- as.POSIXct("2016-07-01 UTC", tz="UTC", origin="1970-01-01")
data$question_creation_date <- as.POSIXct(data$question_creation_date, tz="UTC", origin="1970-01-01")
question_age <- difftime(now, data$question_creation_date, units="days")
data[["question_age"]] <- question_age
data$answer_creation_date <- as.POSIXct(data$answer_creation_date, tz="UTC", origin="1970-01-01")
answer_age <- difftime(now, data$answer_creation_date, units="days")
data[["answer_age"]] <- answer_age

#answers_filter <- grepl("http://stackoverflow.com/a/[0-9]+", data$url, per=TRUE)
#questions_filter <- grepl("http://stackoverflow.com/q/[0-9]+", data$url, per=TRUE)

nrow(data)
# => 7596

summary(data$line_count)
# Min. 1st Qu.  Median    Mean 3rd Qu.    Max. 
# 1.000   1.000   1.000   1.454   1.000  43.000 

table(data$line_count)
#    1    2    3    4    5    6    7    8    9   10   11   12   13   14   15   17   20   22   30   31   34   39   42   43 
# 5919 1063  280  142   77   28   21   10   17    9    5    3    5    1    4    2    1    3    1    1    1    1    1    1 

barplot(table(data$line_count))

d <- density(data$line_count)
plot(d)
polygon(d, col="gray", border="black")

quantile(data$line_count, c(0.75, 0.95))
# 75% 95%
#  1   3 
length(data$line_count[data$line_count<=1])/length(data$line_count)*100
# 77.92259
length(data$line_count[data$line_count<=3])/length(data$line_count)*100
# 95.60295

barplot(sort(data$question_view_count, decreasing=TRUE))
barplot(sort(data$question_score, decreasing=TRUE))
barplot(sort(as.numeric(data$question_age), decreasing=TRUE))

lc_1 <- data[data$line_count==1,]
nrow(lc_1) # 5919
barplot(sort(lc_1$question_view_count, decreasing=TRUE))
barplot(sort(lc_1$question_score, decreasing=TRUE))
barplot(sort(as.numeric(lc_1$question_age), decreasing=TRUE))

lc_2_3 <- data[data$line_count==2 | data$line_count==3,]
nrow(lc_2_3) # 1343
barplot(sort(lc_2_3$question_view_count, decreasing=TRUE))
barplot(sort(lc_2_3$question_score, decreasing=TRUE))
barplot(sort(as.numeric(lc_2_3$question_age), decreasing=TRUE))

lc_4 <- data[data$line_count>3,]
nrow(lc_4) # 334
barplot(sort(lc_4$question_view_count, decreasing=TRUE))
barplot(sort(lc_4$question_score, decreasing=TRUE))
barplot(sort(as.numeric(lc_4$question_age), decreasing=TRUE))

library(effsize)
wilcox.test(as.numeric(lc_2_3$question_view_count), as.numeric(lc_4$question_view_count), alternative="two.sided", paired=F, correct=T) # Wilcoxon rank sum test (equivalent to the Mann-Whitney test), exact distribution (Streitberg-Roehmel algorithm) => handles ties
cliff.delta(as.numeric(lc_2_3$question_view_count), as.numeric(lc_4$question_view_count))
cohen.d(as.numeric(lc_2_3$question_view_count), as.numeric(lc_4$question_view_count))
wilcox.test(as.numeric(lc_2_3$question_score), as.numeric(lc_4$question_score), alternative="two.sided", paired=F, correct=T) # Wilcoxon rank sum test (equivalent to the Mann-Whitney test), exact distribution (Streitberg-Roehmel algorithm) => handles ties
cliff.delta(as.numeric(lc_2_3$question_score), as.numeric(lc_4$question_score))
cohen.d(as.numeric(lc_2_3$question_score), as.numeric(lc_4$question_score))
wilcox.test(as.numeric(lc_2_3$question_age), as.numeric(lc_4$question_age), alternative="two.sided", paired=F, correct=T) # Wilcoxon rank sum test (equivalent to the Mann-Whitney test), exact distribution (Streitberg-Roehmel algorithm) => handles ties
cliff.delta(as.numeric(lc_2_3$question_age), as.numeric(lc_4$question_age))
cohen.d(as.numeric(lc_2_3$question_age), as.numeric(lc_4$question_age))

library(vioplot)
vioplot(lc_4$question_view_count,
        lc_2_3$question_view_count,
        lc_1$question_view_count,
        data$question_view_count,
        col="gray")
vioplot(lc_4$question_score,
        lc_2_3$question_score,
        lc_1$question_score,
        data$question_score,
        col="gray")
vioplot(lc_4$question_age,
        lc_2_3$question_age,
        lc_1$question_age,
        data$question_age,
        col="gray")

barplot(sort(lc_4$question_view_count, decreasing=TRUE))
barplot(sort(lc_4$question_score, decreasing=TRUE))
barplot(sort(as.numeric(lc_4$question_age), decreasing=TRUE))

# Correlations
corr_vars <- data[,c(2, 7, 10, 32)]
corr_vars$question_age <- as.numeric(corr_vars$question_age)

library(corrgram)
corrgram(corr_vars, order=FALSE, lower.panel=panel.shade, upper.panel=panel.conf, cor.method="pearson")

corr_vars_age_norm <- corr_vars
corr_vars_age_norm$question_view_count <- corr_vars$question_view_count/corr_vars$question_age
corr_vars_age_norm$question_score <- corr_vars$question_score/corr_vars$question_age
corrgram(corr_vars_age_norm, order=FALSE, lower.panel=panel.shade, upper.panel=panel.conf, cor.method="pearson")

corr_vars_lc_4 <- lc_4[,c(2, 7, 10, 32)]
corr_vars_lc_4$question_age <- as.numeric(corr_vars_lc_4$question_age)
corrgram(corr_vars_lc_4, order=FALSE, lower.panel=panel.shade, upper.panel=panel.conf, cor.method="pearson")

library(Hmisc) # for rcorr
rcorr(as.matrix(corr_vars), type="pearson") # pariwise deletion
rcorr(as.matrix(corr_vars_lc_4), type="pearson") # pariwise deletion

# see http://stats.stackexchange.com/a/70807, http://stats.stackexchange.com/a/180249, http://stackoverflow.com/a/15779107
normalize <- function(x, min_x, max_x) {
  return ((x-min_x) / (max_x-min_x))
}

normalize_df <- function(df) {
  result <- NULL
  for (i in names(df)) {
    min_i <- min(df[,i])
    max_i <- max(df[,i])
    norm_i <- unlist(as.numeric(lapply(df[,i], function(x) {
      normalize(x, min_i, max_i)
    })))
    if (is.null(result)) {
      result <- norm_i
    } else {
      result <- cbind(result, norm_i)
    }
  }
  result <- data.frame(result)
  names(result) <- names(df)
  return(result)
}

corr_vars_norm <- normalize_df(corr_vars)
library(car)
scatterplotMatrix(~corr_vars_norm$line_count+corr_vars_norm$question_view_count+corr_vars_norm$question_score+corr_vars_norm$question_age, data=corr_vars_norm)

corr_vars_lc_4_norm <- normalize_df(corr_vars_lc_4)
scatterplotMatrix(~corr_vars_lc_4_norm$line_count+corr_vars_lc_4_norm$question_view_count+corr_vars_lc_4_norm$question_score+corr_vars_lc_4_norm$question_age, data=corr_vars_lc_4_norm)
