import argparse
import logging
import os

from ghapi.gh_file import GhFileList
from soapi.so_url import SoUrlList

# get global logger
logger = logging.getLogger('ghapi_logger')


def get_argument_parser():
    arg_parser = argparse.ArgumentParser(
        description='Retrieve commits with SO URL using the GitHub API'
    )
    arg_parser.add_argument(
        '-i', '--start-index',
        type=int,
        required=False,
        default=0,
        help='start index in URL list (default: 0)',
        dest='start_index'
    )
    arg_parser.add_argument(
        '-c', '--chunk-size',
        type=int,
        required=False,
        default=2000,
        help='chunk size for this call (default: 2000)',
        dest='chunk_size'
    )
    arg_parser.add_argument(
        '-id', '--import-dir',
        required=True,
        help='path to import directory for CSV file',
        dest='import_dir'
    )
    arg_parser.add_argument(
        '-ed', '--export-dir',
        required=True,
        help='path to export directory for CSV and JSON data',
        dest='export_dir'
    )
    arg_parser.add_argument(
        '-n', '--name',
        required=True,
        help='name used for import and export',
        dest='name'
    )
    arg_parser.add_argument(
        '-s', '--skip-header',
        required=False,
        default=True,
        help='skip header of CSV file? (default: True)',
        dest='skip_header'
    )
    arg_parser.add_argument(
        '-d', '--delimiter',
        required=False,
        default=',',
        help='delimiter in csv file (default: \',\')',
        dest='delimiter'
    )
    return arg_parser


def main():
    # parse command line arguments
    parser = get_argument_parser()
    args = parser.parse_args()

    # please insert gh access token here
    gh_access_token = ""
    
    file_list = GhFileList(args.name, args.start_index, args.chunk_size)

    file_path = os.path.join(
        args.import_dir,
        '{0}.csv'.format(args.name)
    )

    file_list.read_from_csv(file_path, args.skip_header, args.delimiter)
    file_list.retrieve_gh_data(gh_access_token)
    file_list.write_to_csv(args.export_dir, args.delimiter)

if __name__ == '__main__':
    main()